@@ -19,17 +19,18 @@
 #############################################################################################
 
 from __future__ import division, print_function
-from PyQt5 import QtCore, QtWidgets
+from PyQt5 import QtCore, QtWidgets, QtGui
 import numpy as np
 import sys, os
 import pyqtgraph as pg
 import gc
+from pathlib import Path
 
 from .dialogs import ControlPanel, PreprocessingWidget, SaveWidget, EditMetadataWidget
 from .gui_utils import sibling_path, pg_point_roi, LQCollection, datacube_selector
-from ..file.io.read import read
-from ..file.io.native import save, is_py4DSTEM_file
-from ..file.datastructure.datacube import DataCube
+from ..io.read import read
+from ..io.native import save, is_py4DSTEM_file
+from ..io.datastructure.datacube import DataCube
 from .strain import *
 
 import IPython
@@ -66,6 +67,10 @@ class DataViewer(QtWidgets.QMainWindow):
         self.main_window = QtWidgets.QWidget()
         self.main_window.setWindowTitle("py4DSTEM")
 
+        icon = QtGui.QIcon(str(Path(__file__).parent.absolute() / "logo.png"))
+        self.setWindowIcon(icon)
+        self.qtapp.setWindowIcon(icon)
+
         # Set up sub-windows and arrange into primary py4DSTEM window
         self.setup_diffraction_space_widget()
         self.setup_real_space_widget()
@@ -84,6 +89,14 @@ class DataViewer(QtWidgets.QMainWindow):
         self.diffraction_space_widget.ui.normDivideRadio.setChecked(True)
         self.diffraction_space_widget.normRadioChanged()
 
+        if len(argv) > 1:
+            path = os.path.abspath(argv[1])
+            try:
+                self.settings.data_filename.val = path
+                self.load_file()
+            except Exception:
+                print(f"Tried to set file to {path} but something went wrong...")
+
 
     ###############################################
     ############ Widget setup methods #############
@@ -297,7 +310,7 @@ class DataViewer(QtWidgets.QMainWindow):
             if is_py4DSTEM_file(fname):
                 self.datacube = datacube_selector(fname)
             else:
-                self.datacube,_ = read(fname)
+                self.datacube = read(fname)
             if type(self.datacube) == str :
                 self.Unidentified_file(fname)
                 #Reset view
@@ -329,6 +342,8 @@ class DataViewer(QtWidgets.QMainWindow):
         self.control_widget.spinBox_Nx.setMaximum(self.datacube.R_N)
         self.control_widget.spinBox_Ny.setMaximum(self.datacube.R_N)
 
+        self.main_window.setWindowTitle(fname)
+
         return
 
     ############## Preprocess ##############
