@@ -1,19 +1,22 @@
-# Reader for py4DSTEM v0.9 - v0.11 files
+# Reader for py4DSTEM v0.5 files
 
 import h5py
 import numpy as np
-from os.path import splitext, exists
+from os.path import splitext
 from .read_utils import is_py4DSTEM_file, get_py4DSTEM_topgroups, get_py4DSTEM_version, version_is_geq
-from .read_utils_v0_9 import get_py4DSTEM_dataobject_info
+from .read_utils_v0_5 import get_py4DSTEM_dataobject_info
 from ...datastructure import DataCube, CountedDataCube, DiffractionSlice, RealSlice
 from ...datastructure import PointList, PointListArray
 from ....process.utils import tqdmnd
 
-def read_v0_9(fp, **kwargs):
+def read_v0_5(fp, **kwargs):
     """
-    File reader for files written by py4DSTEM v0.9-0.11.  Precise behavior is detemined by which
+    File reader for files written by py4DSTEM v0.5.  Precise behavior is detemined by which
     arguments are passed -- see below.
 
+    ***NOTE: this function has not yet been tested on all legacy py4DSTEM formats. Please report
+    any problems by filing an issue on our github!
+
     Accepts:
         filepath    str or Path     When passed a filepath only, this function checks if the path
                                     points to a valid py4DSTEM file, then prints its contents to screen.
@@ -50,7 +53,6 @@ def read_v0_9(fp, **kwargs):
                                     will return a length two tuple, the first element being a list of 3
                                     DataObject instances and the second a MetaData instance.
     """
-    assert(exists(fp)), "Error: specified filepath does not exist"
     assert(is_py4DSTEM_file(fp)), "Error: {} isn't recognized as a py4DSTEM file.".format(fp)
 
     # For HDF5 files containing multiple valid EMD type 2 files, disambiguate desired data
@@ -70,88 +72,75 @@ def read_v0_9(fp, **kwargs):
             return None,None
 
     version = get_py4DSTEM_version(fp, tg)
-    assert(version_is_geq(version,(0,9,0)) and not version_is_geq(version,(0,12,0))), (
-                                                               "File must be v0.9-0.11")
+    assert(version == (0,5,0)), "File must be v0.5.0."
     _data_id = 'data_id' in kwargs.keys()  # Flag indicating if data was requested
 
-    # If metadata is requested
-    if 'metadata' in kwargs.keys():
-        if kwargs['metadata']:
-            return metadata_from_h5(fp, tg)
-
-    # If data is requested
-    elif 'data_id' in kwargs.keys():
+    # Validate inputs
+    if _data_id:
         data_id = kwargs['data_id']
-        assert(isinstance(data_id,(int,np.int_,str,list,tuple))), "Error: data must be specified with strings or integers only."
-        if not isinstance(data_id,(int,np.int_,str)):
-            assert(all([isinstance(d,(int,np.int_,str)) for d in data_id])), "Error: data must be specified with strings or integers only."
+        assert(isinstance(data_id,(int,str,list,tuple))), "Error: data must be specified with strings or integers only."
+        if not isinstance(data_id,(int,str)):
+            assert(all([isinstance(d,(int,str)) for d in data_id])), "Error: data must be specified with strings or integers only."
 
-        # Parse optional arguments
-        if 'mem' in kwargs.keys():
-            mem = kwargs['mem']
-            assert(mem in ('RAM','MEMMAP'))
-        else:
-            mem='RAM'
-        if 'binfactor' in kwargs.keys():
-            binfactor = kwargs['binfactor']
-            assert(isinstance(binfactor,(int,np.int_)))
-        else:
-            binfactor=1
-        if 'dtype' in kwargs.keys():
-            bindtype = kwargs['dtype']
-            assert(isinstance(bindtype,type))
-        else:
-            bindtype = None
-
-        return get_data(fp,tg,data_id,mem,binfactor,bindtype)
-
-    # If no data is requested
+    # Parse optional arguments
+    if 'mem' in kwargs.keys():
+        mem = kwargs['mem']
+        assert(mem in ('RAM','MEMMAP'))
     else:
+        mem='RAM'
+    if 'binfactor' in kwargs.keys():
+        binfactor = kwargs['binfactor']
+        assert(isinstance(binfactor,int))
+    else:
+        binfactor=1
+    if 'dtype' in kwargs.keys():
+        bindtype = kwargs['dtype']
+        assert(isinstance(bindtype,type))
+    else:
+        bindtype = None
+
+    # Perform requested operations
+    if not _data_id:
         print_py4DSTEM_file(fp,tg)
         return
-
-
-
-
-
-
-
+    else:
+        return get_data(fp,tg,data_id,mem,binfactor,bindtype)
 
 
 ############ Helper functions ############
 
-def print_py4DSTEM_file(filepath,tg):
-    """ Accepts a filepath to a valid py4DSTEM file and prints to screen the file contents.
+def print_py4DSTEM_file(fp,tg):
+    """ Accepts a fp to a valid py4DSTEM file and prints to screen the file contents.
     """
-    info = get_py4DSTEM_dataobject_info(filepath,tg)
+    info = get_py4DSTEM_dataobject_info(fp,tg)
     print("{:10}{:18}{:24}{:54}".format('Index', 'Type', 'Shape', 'Name'))
     print("{:10}{:18}{:24}{:54}".format('-----', '----', '-----', '----'))
     for el in info:
         print("  {:8}{:18}{:24}{:54}".format(str(el['index']),str(el['type']),str(el['shape']),str(el['name'])))
+
     return
 
-def get_data(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a filepath to a valid py4DSTEM file and an int/str/list specifying data, and returns the data.
+def get_data(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a fp to a valid py4DSTEM file and an int/str/list specifying data, and returns the data.
     """
-    if isinstance(data_id,(int,np.int_)):
-        return get_data_from_int(filepath,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
+    if isinstance(data_id,int):
+        return get_data_from_int(fp,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
     elif isinstance(data_id,str):
-        return get_data_from_str(filepath,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
+        return get_data_from_str(fp,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
     else:
-        return get_data_from_list(filepath,tg,data_id)
+        return get_data_from_list(fp,tg,data_id)
 
-def get_data_from_int(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a filepath to a valid py4DSTEM file and an integer specifying data, and returns the data.
+def get_data_from_int(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a fp to a valid py4DSTEM file and an integer specifying data, and returns the data.
     """
-    assert(isinstance(data_id,(int,np.int_)))
-    with h5py.File(filepath,'r') as f:
+    assert(isinstance(data_id,int))
+    with h5py.File(fp,'r') as f:
         grp_dc = f[tg+'/data/datacubes/']
-        grp_cdc = f[tg+'/data/counted_datacubes/']
         grp_ds = f[tg+'/data/diffractionslices/']
         grp_rs = f[tg+'/data/realslices/']
         grp_pl = f[tg+'/data/pointlists/']
         grp_pla = f[tg+'/data/pointlistarrays/']
-        grps = [grp_dc,grp_cdc,grp_ds,grp_rs,grp_pl,grp_pla]
+        grps = [grp_dc,grp_ds,grp_rs,grp_pl,grp_pla]
 
         Ns = np.cumsum([len(grp.keys()) for grp in grps])
         i = np.nonzero(data_id<Ns)[0][0]
@@ -164,26 +153,24 @@ def get_data_from_int(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
 
     return data
 
-def get_data_from_str(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a filepath to a valid py4DSTEM file and a string specifying data, and returns the data.
+def get_data_from_str(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a fp to a valid py4DSTEM file and a string specifying data, and returns the data.
     """
     assert(isinstance(data_id,str))
-    with h5py.File(filepath,'r') as f:
+    with h5py.File(fp,'r') as f:
         grp_dc = f[tg+'/data/datacubes/']
-        grp_cdc = f[tg+'/data/counted_datacubes/']
         grp_ds = f[tg+'/data/diffractionslices/']
         grp_rs = f[tg+'/data/realslices/']
         grp_pl = f[tg+'/data/pointlists/']
         grp_pla = f[tg+'/data/pointlistarrays/']
-        grps = [grp_dc,grp_cdc,grp_ds,grp_rs,grp_pl,grp_pla]
+        grps = [grp_dc,grp_ds,grp_rs,grp_pl,grp_pla]
 
         l_dc = list(grp_dc.keys())
-        l_cdc = list(grp_cdc.keys())
         l_ds = list(grp_ds.keys())
         l_rs = list(grp_rs.keys())
         l_pl = list(grp_pl.keys())
         l_pla = list(grp_pla.keys())
-        names = l_dc+l_cdc+l_ds+l_rs+l_pl+l_pla
+        names = l_dc+l_ds+l_rs+l_pl+l_pla
 
         inds = [i for i,name in enumerate(names) if name==data_id]
         assert(len(inds)!=0), "Error: no data named {} found.".format(data_id)
@@ -199,17 +186,17 @@ def get_data_from_str(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
 
     return data
 
-def get_data_from_list(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a filepath to a valid py4DSTEM file and a list or tuple specifying data, and returns the data.
+def get_data_from_list(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a fp to a valid py4DSTEM file and a list or tuple specifying data, and returns the data.
     """
     assert(isinstance(data_id,(list,tuple)))
-    assert(all([isinstance(d,(int,np.int_,str)) for d in data_id]))
+    assert(all([isinstance(d,(int,str)) for d in data_id]))
     data = []
     for el in data_id:
-        if isinstance(el,(int,np.int_)):
-            data.append(get_data_from_int(filepath,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
+        if isinstance(el,int):
+            data.append(get_data_from_int(fp,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
         elif isinstance(el,str):
-            data.append(get_data_from_str(filepath,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
+            data.append(get_data_from_str(fp,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
         else:
             raise Exception("Data must be specified with strings or integers only.")
     return data
@@ -239,7 +226,7 @@ def get_datacube_from_grp(g,mem='RAM',binfactor=1,bindtype=None):
         and returns a DataCube.
     """
     # TODO: add memmapping, binning
-    data = np.array(g['data'])
+    data = np.array(g['datacube'])
     name = g.name.split('/')[-1]
     return DataCube(data=data,name=name)
 
@@ -253,7 +240,7 @@ def get_diffractionslice_from_grp(g):
     """ Accepts an h5py Group corresponding to a diffractionslice in an open, correctly formatted H5 file,
         and returns a DiffractionSlice.
     """
-    data = np.array(g['data'])
+    data = np.array(g['diffractionslice'])
     name = g.name.split('/')[-1]
     Q_Nx,Q_Ny = data.shape[:2]
     if len(data.shape)==2:
@@ -270,7 +257,7 @@ def get_realslice_from_grp(g):
     """ Accepts an h5py Group corresponding to a realslice in an open, correctly formatted H5 file,
         and returns a RealSlice.
     """
-    data = np.array(g['data'])
+    data = np.array(g['realslice'])
     name = g.name.split('/')[-1]
     R_Nx,R_Ny = data.shape[:2]
     if len(data.shape)==2:
@@ -290,17 +277,17 @@ def get_pointlist_from_grp(g):
     name = g.name.split('/')[-1]
     coordinates = []
     coord_names = list(g.keys())
-    length = len(g[coord_names[0]+'/data'])
+    length = len(g[coord_names[0]+'/pointlist'])
     if length==0:
         for coord in coord_names:
             coordinates.append((coord,None))
     else:
         for coord in coord_names:
-            dtype = type(g[coord+'/data'][0])
+            dtype = type(g[coord+'/pointlist'][0])
             coordinates.append((coord, dtype))
     data = np.zeros(length,dtype=coordinates)
     for coord in coord_names:
-        data[coord] = np.array(g[coord+'/data'])
+        data[coord] = np.array(g[coord+'/pointlist'])
     return PointList(data=data,coordinates=coordinates,name=name)
 
 def get_pointlistarray_from_grp(g):
@@ -308,13 +295,36 @@ def get_pointlistarray_from_grp(g):
         and returns a PointListArray.
     """
     name = g.name.split('/')[-1]
-    dset = g['data']
-    shape = g['data'].shape
-    coordinates = g['data'][0,0].dtype
+    l = list(g)
+    ar = np.array([l[i].split('_') for i in range(len(l))]).astype(int)
+    shape = (np.max(ar[:,0])+1,np.max(ar[:,1])+1)
+    coord_names = list(g['0_0'])
+    N = len(coord_names)
+    coord_types = [type(np.array(g['0_0/'+coord_names[i]+'/pointlistarray'])[0]) for i in range(N)]
+    coordinates = [(coord_names[i],coord_types[i]) for i in range(N)]
     pla = PointListArray(coordinates=coordinates,shape=shape,name=name)
-    for (i,j) in tqdmnd(shape[0],shape[1],desc="Reading PointListArray",unit="PointList"):
-        pla.get_pointlist(i,j).add_dataarray(dset[i,j])
+    for (i,j) in tqdmnd(range(shape[0]),range(shape[1]),desc="Reading PointListArray",unit="PointList"):
+        g_pl = g[str(i)+'_'+str(j)]
+        L = len(np.array(g_pl[coordinates[0][0]+'/pointlistarray']))
+        data = np.zeros(L,dtype=coordinates)
+        for i in range(N):
+            coord = coordinates[i][0]
+            data[coord] = np.array(g_pl[coord+'/pointlistarray'])
+        pla.get_pointlist(i,j).add_dataarray(data)
     return pla
 
 
+########### Metadata and log ############
+
+def get_metadata(fp,tg):
+    """ Accepts a fp to a valid py4DSTEM file, and return a dictionary with its metadata.
+    """
+    return #TODO
+
+def write_log(fp,tg):
+    """ Accepts a fp to a valid py4DSTEM file, then prints its processing log to splitext(fp)[0]+'.log'.
+    """
+    return #TODO
+
+
 
