@@ -10,13 +10,16 @@ from mpl_toolkits.axes_grid1 import make_axes_locatable
 from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
 import matplotlib.font_manager as fm
 
+from .multicorr import upsampled_correlation
+
 try:
     from IPython.display import clear_output
 except ImportError:
     def clear_output(wait=True):
         pass
 
-def plot(img, title='Image', savePath=None, cmap='inferno', show=True, vmax=None, figsize=(10, 10), scale=None):
+def plot(img, title='Image', savePath=None, cmap='inferno', show=True, vmax=None,
+                                                        figsize=(10, 10), scale=None):
     fig, ax = plt.subplots(figsize=figsize)
     im = ax.imshow(img, interpolation='nearest', cmap=plt.cm.get_cmap(cmap), vmax=vmax)
     divider = make_axes_locatable(ax)
@@ -56,13 +59,12 @@ def sector_mask(shape, centre, radius, angle_range=(0, 360)):
     Return a boolean mask for a circular sector. The start/stop angles in
     `angle_range` should be given in clockwise order.
 
-    :param shape: 2D shape of the mask
-    :param centre: 2D center of the circular sector
-    :param radius: radius of the circular mask
-    :param angle_range: angular range of the circular mask
-    :return:
+    Args:
+        shape: 2D shape of the mask
+        centre: 2D center of the circular sector
+        radius: radius of the circular mask
+        angle_range: angular range of the circular mask
     """
-
     x, y = np.ogrid[:shape[0], :shape[1]]
     cx, cy = centre
     tmin, tmax = np.deg2rad(angle_range)
@@ -94,10 +96,10 @@ def get_qx_qy_1d(M, dx=[1, 1], fft_shifted=False):
     Generates 1D Fourier coordinates for a (Nx,Ny)-shaped 2D array.
     Specifying the dx argument sets a unit size.
 
-    :param M: (2,) shape of the returned array
-    :param dx: (2,) tuple, pixel size
-    :param fft_shifted: True if result should be fft_shifted to have the origin in the center of the array
-    :return:
+    Args:
+        M: (2,) shape of the returned array
+        dx: (2,) tuple, pixel size
+        fft_shifted: True if result should be fft_shifted to have the origin in the center of the array
     """
 
     qxa = fftfreq(M[0], dx[0])
@@ -136,11 +138,13 @@ def get_shift(ar1, ar2, corrPower=1):
     pixel resolution. corrPower specifies the cross correlation power, with 1 corresponding to a
     cross correlation and 0 a phase correlation.
 
-	Inputs:
-		ar1,ar2     2D ndarrays
-        corrPower   float between 0 and 1, inclusive. 1=cross correlation, 0=phase correlation
-	Outputs:
-		shiftx,shifty - relative image shift, in pixels
+	Args:
+		ar1,ar2 (2D ndarrays):
+        corrPower (float between 0 and 1, inclusive): 1=cross correlation, 0=phase
+            correlation
+
+    Returns:
+		(2-tuple): (shiftx,shifty) - the relative image shift, in pixels
     """
     cc = get_cross_correlation(ar1, ar2, corrPower)
     xshift, yshift = np.unravel_index(np.argmax(cc), ar1.shape)
@@ -161,26 +165,40 @@ def get_shifted_ar(ar, xshift, yshift):
     return shifted_ar
 
 
-def get_cross_correlation(ar, kernel, corrPower=1):
+def get_cross_correlation(ar, kernel, corrPower=1, returnval='cc'):
     """
     Calculates the cross correlation of ar with kernel.
     corrPower specifies the correlation type, where 1 is a cross correlation, 0 is a phase
     correlation, and values in between are hybrids.
+
+    The return value depends on the argument ``returnval``. If return=='cc' (default),
+    returns the real part of the cross correlation in real space.  If return=='fourier',
+    returns the output in Fourier space, before taking the inverse transform.
     """
-    m = np.fft.fft2(ar) * np.conj(np.fft.fft2(kernel))
-    return np.real(np.fft.ifft2(np.abs(m) ** (corrPower) * np.exp(1j * np.angle(m))))
+    assert(returnval in ('cc','fourier'))
+    fourierkernel = np.conj(np.fft.fft2(kernel))
+    return get_cross_correlation_fk(ar, fourierkernel, corrPower=corrPower, returnval=returnval)
 
 
-def get_cross_correlation_fk(ar, fourierkernel, corrPower=1):
+def get_cross_correlation_fk(ar, fourierkernel, corrPower=1, returnval='cc'):
     """
     Calculates the cross correlation of ar with fourierkernel.
     Here, fourierkernel = np.conj(np.fft.fft2(kernel)); speeds up computation when the same
     kernel is to be used for multiple cross correlations.
     corrPower specifies the correlation type, where 1 is a cross correlation, 0 is a phase
     correlation, and values in between are hybrids.
+
+    The return value depends on the argument ``returnval``. If return=='cc' (default),
+    returns the real part of the cross correlation in real space.  If return=='fourier',
+    returns the output in Fourier space, before taking the inverse transform.
     """
+    assert(returnval in ('cc','fourier'))
     m = np.fft.fft2(ar) * fourierkernel
-    return np.real(np.fft.ifft2(np.abs(m) ** (corrPower) * np.exp(1j * np.angle(m))))
+    ccc = np.abs(m)**(corrPower) * np.exp(1j*np.angle(m))
+    if returnval=='fourier':
+        return ccc
+    else:
+        return np.real(np.fft.ifft2(ccc))
 
 
 def get_CoM(ar):
@@ -207,30 +225,41 @@ def get_maximal_points(ar):
 
 
 def get_maxima_2D(ar, sigma=0, edgeBoundary=0, minSpacing=0, minRelativeIntensity=0,
-                  relativeToPeak=0, maxNumPeaks=0, subpixel=True):
+                  relativeToPeak=0, maxNumPeaks=0, subpixel='poly', ar_FT=None, upsample_factor=16):
     """
     Finds the indices where the 2D array ar is a local maximum.
     Optional parameters allow blurring of the array and filtering of the output;
     setting each of these to 0 (default) turns off these functions.
 
-    Accepts:
-        ar                      (ndarray) a 2D array
-        sigma                   (float) guassian blur std to applyu to ar before finding the maxima
-        edgeBoundary            (int) ignore maxima within edgeBoundary of the array edge
-        minSpacing              (float) if two maxima are found within minSpacing, the dimmer one
-                                is removed
-        minRelativeIntensity    (float) maxima dimmer than minRelativeIntensity compared to the
-                                relativeToPeak'th brightest maximum are removed
-        relativeToPeak          (int) 0=brightest maximum. 1=next brightest, etc.
-        maxNumPeaks             (int) return only the first maxNumPeaks maxima
-        subpixel                (bool) if False, return locally maximal pixels.
-                                if True, perform subpixel fitting
+    Args:
+        ar (ndarray): a 2D array
+        sigma (float): guassian blur std to applyu to ar before finding the maxima
+        edgeBoundary (int): ignore maxima within edgeBoundary of the array edge
+        minSpacing (float): if two maxima are found within minSpacing, the dimmer one
+            is removed
+        minRelativeIntensity (float): maxima dimmer than minRelativeIntensity compared
+            to the relativeToPeak'th brightest maximum are removed
+        relativeToPeak (int): 0=brightest maximum. 1=next brightest, etc.
+        maxNumPeaks (int): return only the first maxNumPeaks maxima
+        subpixel (str): Whether to use subpixel fitting, and which algorithm to use.
+            Must be in ('none','poly','multicorr').
+                * 'none': performs no subpixel fitting
+                * 'poly': polynomial interpolation of correlogram peaks (default)
+                * 'multicorr': uses the multicorr algorithm with DFT upsampling
+        ar_FT (None or complex array): if subpixel=='multicorr' the fourier transform of
+            the image is required.  It may be passed here as a complex array.  Otherwise,
+            if ar_FT is None, it is computed
+        upsample_factor (int): required iff subpixel=='multicorr'
 
-    Returns
-        maxima_x                (ndarray) x-coords of the local maximum, sorted by intensity.
-        maxima_y                (ndarray) y-coords of the local maximum, sorted by intensity.
-        maxima_intensity        (ndarray) intensity of the local maxima
+    Returns:
+        (3-tuple): A 3-tuple containing:
+
+            * **maxima_x** *(ndarray)*: x-coords of the local maximum, sorted by intensity.
+            * **maxima_y** *(ndarray)*: y-coords of the local maximum, sorted by intensity.
+            * **maxima_intensity** *(ndarray)*: intensity of the local maxima
     """
+    assert subpixel in [ 'none', 'poly', 'multicorr' ], "Unrecognized subpixel option {}, subpixel must be 'none', 'poly', or 'multicorr'".format(subpixel)
+
     # Get maxima
     ar = gaussian_filter(ar, sigma)
     maxima_bool = get_maximal_points(ar)
@@ -281,8 +310,9 @@ def get_maxima_2D(ar, sigma=0, edgeBoundary=0, minSpacing=0, minRelativeIntensit
             if len(maxima) > maxNumPeaks:
                 maxima = maxima[:maxNumPeaks]
 
-        # Subpixel fitting - fit 1D parabolas in x and y to 3 points (maximum, +/- 1 pixel)
-        if subpixel is True:
+        # Subpixel fitting 
+        # For all subpixel fitting, first fit 1D parabolas in x and y to 3 points (maximum, +/- 1 pixel)
+        if subpixel != 'none':
             for i in range(len(maxima)):
                 Ix1_ = ar[int(maxima['x'][i]) - 1, int(maxima['y'][i])]
                 Ix0 = ar[int(maxima['x'][i]), int(maxima['y'][i])]
@@ -295,6 +325,21 @@ def get_maxima_2D(ar, sigma=0, edgeBoundary=0, minSpacing=0, minRelativeIntensit
                 maxima['x'][i] += deltax
                 maxima['y'][i] += deltay
                 maxima['intensity'][i] = linear_interpolation_2D(ar, maxima['x'][i], maxima['y'][i])
+        # Further refinement with fourier upsampling
+        if subpixel == 'multicorr':
+            if ar_FT is None:
+                ar_FT = np.fft.fft2(ar)
+            for ipeak in range(len(maxima['x'])):
+                xyShift = np.array((maxima['x'][ipeak],maxima['y'][ipeak]))
+                # we actually have to lose some precision and go down to half-pixel
+                # accuracy. this could also be done by a single upsampling at factor 2
+                # instead of get_maxima_2D.
+                xyShift[0] = np.round(xyShift[0] * 2) / 2
+                xyShift[1] = np.round(xyShift[1] * 2) / 2
+
+                subShift = upsampled_correlation(ar_FT,upsample_factor,xyShift)
+                maxima['x'][ipeak]=subShift[0]
+                maxima['y'][ipeak]=subShift[1]
 
     return maxima['x'], maxima['y'], maxima['intensity']
 
@@ -305,16 +350,17 @@ def get_maxima_1D(ar, sigma=0, minSpacing=0, minRelativeIntensity=0, relativeToP
     Optional parameters allow blurring the array and filtering the output;
     setting each to 0 (default) turns off these functions.
 
-    Accepts:
-        ar                    a 1D array
-        sigma                 gaussian blur std to apply to ar before finding maxima
-        minSpacing            if two maxima are found within minSpacing, the dimmer one is removed
-        minRelativeIntensity  maxima dimmer than minRelativeIntensity compared to the
-                              relativeToPeak'th brightest maximum are removed
-        relativeToPeak        0=brightest maximum. 1=next brightest, etc.
+    Args:
+        ar (1D array):
+        sigma (number): gaussian blur std to apply to ar before finding maxima
+        minSpacing (number): if two maxima are found within minSpacing, the dimmer one
+            is removed
+        minRelativeIntensity (number): maxima dimmer than minRelativeIntensity compared
+            to the relativeToPeak'th brightest maximum are removed
+        relativeToPeak (int): 0=brightest maximum. 1=next brightest, etc.
 
     Returns:
-        An array of indices where ar is a local maximum, sorted by intensity.
+        (array of ints): An array of indices where ar is a local maximum, sorted by intensity.
     """
     assert len(ar.shape) == 1, "ar must be 1D"
     assert isinstance(relativeToPeak, (int, np.integer)), "relativeToPeak must be an int"
@@ -352,7 +398,8 @@ def get_maxima_1D(ar, sigma=0, minSpacing=0, minRelativeIntensity=0, relativeToP
 
 def linear_interpolation_1D(ar, x):
     """
-    Calculates the 1D linear interpolation of array ar at position x using the two nearest elements.
+    Calculates the 1D linear interpolation of array ar at position x using the two
+    nearest elements.
     """
     x0, x1 = int(np.floor(x)), int(np.ceil(x))
     dx = x - x0
@@ -361,8 +408,8 @@ def linear_interpolation_1D(ar, x):
 
 def linear_interpolation_2D(ar, x, y):
     """
-    Calculates the 2D linear interpolation of array ar at position x,y using the four nearest
-    array elements.
+    Calculates the 2D linear interpolation of array ar at position x,y using the four
+    nearest array elements.
     """
     x0, x1 = int(np.floor(x)), int(np.ceil(x))
     y0, y1 = int(np.floor(y)), int(np.ceil(y))
@@ -374,8 +421,8 @@ def linear_interpolation_2D(ar, x, y):
 
 def add_to_2D_array_from_floats(ar, x, y, I):
     """
-    Adds the values I to array ar, distributing the value between the four pixels nearest (x,y) using
-    linear interpolation.  Inputs (x,y,I) may be floats or arrays of floats.
+    Adds the values I to array ar, distributing the value between the four pixels nearest
+    (x,y) using linear interpolation.  Inputs (x,y,I) may be floats or arrays of floats.
     """
     Nx, Ny = ar.shape
     x0, x1 = (np.floor(x)).astype(int), (np.ceil(x)).astype(int)
@@ -394,14 +441,15 @@ def print_progress_bar(iteration, total, prefix='', suffix='', decimals=1,
                        length=100, fill='*'):
     """
     Call in a loop to create terminal progress bar
-    @params:
-        iteration   - Required  : current iteration (Int)
-        total       - Required  : total iterations (Int)
-        prefix      - Optional  : prefix string (Str)
-        suffix      - Optional  : suffix string (Str)
-        decimals    - Optional  : positive number of decimals in percent complete (Int)
-        length      - Optional  : character length of bar (Int)
-        fill        - Optional  : bar fill character (Str)
+
+    Args:
+        iteration (int): current iteration
+        total (int): total iterations
+        prefix (str, optional): prefix string
+        suffix  (str, optional): suffix string
+        decimals (int, optional): positive number of decimals in percent complete
+        length (int, optional): character length of bar
+        fill (str, optional): bar fill character
     """
     percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
     filledLength = int(length * iteration // total)
@@ -417,14 +465,14 @@ def bin2D(array, factor, dtype=np.float64):
     """
     Bin a 2D ndarray by binfactor.
 
-    Accepts:
-        array       a 2D numpy array
-        factor      (int) the binning factor
-        dtype       (numpy dtype) datatype for binned array.
-                    defalt is numpy default for np.zeros()
+    Args:
+        array (2D numpy array):
+        factor (int): the binning factor
+        dtype (numpy dtype): datatype for binned array. default is numpy default for
+            np.zeros()
 
     Returns:
-        binned_ar   the binned array
+        the binned array
     """
     x, y = array.shape
     binx, biny = x // factor, y // factor
@@ -443,26 +491,26 @@ def bin2D(array, factor, dtype=np.float64):
 
 def get_voronoi_vertices(voronoi, nx, ny, dist=10):
     """
-    From a scipy.spatial.Voronoi instance, return a list of ndarrays, where each array is shape
-    (N,2) and contains the (x,y) positions of the vertices of a voronoi region.
+    From a scipy.spatial.Voronoi instance, return a list of ndarrays, where each array
+    is shape (N,2) and contains the (x,y) positions of the vertices of a voronoi region.
 
-    The problem this function solves is that in a Voronoi instance, some vertices outside the
-    field of view of the tesselated region are left unspecified; only the existence of a point
-    beyond the field is referenced (which may or may not be 'at infinity'). This function
-    specifies all points, such that the vertices and edges of the tesselation may be directly
-    laid over data.
+    The problem this function solves is that in a Voronoi instance, some vertices outside
+    the field of view of the tesselated region are left unspecified; only the existence
+    of a point beyond the field is referenced (which may or may not be 'at infinity').
+    This function specifies all points, such that the vertices and edges of the
+    tesselation may be directly laid over data.
 
-    Accepts:
-        voronoi     (scipy.spatial.Voronoi) the voronoi tesselation
-        nx          (int) the x field-of-view of the tesselated region
-        ny          (int) the y field-of-view of the tesselated region
-        dist (opt)  (float) place new vertices by extending new voronoi edges outside the frame
-                    by a distance of this factor times the distance of its known vertex from
-                    the frame edge
+    Args:
+        voronoi (scipy.spatial.Voronoi): the voronoi tesselation
+        nx (int): the x field-of-view of the tesselated region
+        ny (int): the y field-of-view of the tesselated region
+        dist (float, optional): place new vertices by extending new voronoi edges outside
+            the frame by a distance of this factor times the distance of its known vertex
+            from the frame edge
 
-    Returns
-        vertex_list (list of ndarrays of shape (N,2)) the (x,y) coords of the vertices of each
-                    voronoi region
+    Returns:
+        (list of ndarrays of shape (N,2)): the (x,y) coords of the vertices of each
+        voronoi region
     """
     assert isinstance(voronoi, Voronoi), "voronoi must be a scipy.spatial.Voronoi instance"
 
@@ -550,11 +598,11 @@ def get_voronoi_vertices(voronoi, nx, ny, dist=10):
 
 def get_ewpc_filter_function(Q_Nx, Q_Ny):
     '''
-    Returns a function for computing the exit wave power cepstrum of a diffraction pattern
-    using a Hanning window. This can be passed as the filter_function in the Bragg disk
-    detection functions (with the probe an array of ones) to find the lattice vectors
-    by the EWPC method (but be careful as the lengths are now in realspace units!)
-    See https://arxiv.org/abs/1911.00984
+    Returns a function for computing the exit wave power cepstrum of a diffraction
+    pattern using a Hanning window. This can be passed as the filter_function in the
+    Bragg disk detection functions (with the probe an array of ones) to find the lattice
+    vectors by the EWPC method (but be careful as the lengths are now in realspace
+    units!) See https://arxiv.org/abs/1911.00984
     '''
     h = np.hanning(Q_Nx)[:,np.newaxis] * np.hanning(Q_Ny)[np.newaxis,:]
     return lambda x: np.abs(np.fft.fftshift(np.fft.fft2(h*np.log(np.maximum(x,0.01)))))**2
